package com.ruoyi.benyun.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.ruoyi.benyun.entity.vo.ChannelAnalyseAceeptVo;
import com.ruoyi.benyun.entity.vo.ChannelMapAceeptVo;
import com.ruoyi.benyun.entity.vo.TagAnalyse;
import com.ruoyi.benyun.mapper.WdInfoDao;
import com.ruoyi.benyun.service.ChannelAnalyseService;
import com.ruoyi.benyun.entity.Brand;
import com.ruoyi.benyun.entity.WdInfo;
import com.ruoyi.benyun.mapper.BrandMapper;
import com.ruoyi.benyun.mapper.WdTopologicalInfoDao;
import com.ruoyi.benyun.utils.WdRedisStoreage;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.*;

@Service
public class ChannelAnalyseServiceImpl implements ChannelAnalyseService {

    @Autowired
    WdInfoDao wdInfoDao;

    @Autowired
    @Qualifier("cityTierMap")
    HashMap<String, String> cityTierMap;

    @Autowired
    BrandMapper brandMapper;

    @Autowired
    WdTopologicalInfoDao wdTopologicalInfoDao;

    @Autowired
    @Qualifier("typeByMap")
    private HashMap<String, String> typeByMap;

    @Autowired
    private WdRedisStoreage wdRedisStoreage;

    @Autowired
    @Qualifier("cityTierListMap")
    HashMap<String,List<String>> cityTierListMap;

    @Override
    public HashMap cityTier(ChannelMapAceeptVo channelMapAceeptVo) {
        //1.统计结果
        HashMap<String, Long> result = new HashMap<>();
        result.put("一线", 0L);
        result.put("新一线", 0L);
        result.put("二线", 0L);
        result.put("三线", 0L);
        result.put("四线", 0L);
        result.put("五线", 0L);
        result.put("其他", 0L);


        //1.获取所有符合条件的网点信息
        List<WdInfo> wdInfos = getQueryWd(channelMapAceeptVo);
        for (WdInfo wdInfo : wdInfos) {
            String addrCode = wdInfo.getAddrCode().substring(0,4);
            if (cityTierMap.containsKey(addrCode)) {
                result.put(cityTierMap.get(addrCode), result.get(cityTierMap.get(addrCode)) + 1);
            } else
                result.put("其他", result.get("其他") + 1);
        }


        return result;
    }

    @Override
    public PageInfo<Brand> brandList(ChannelAnalyseAceeptVo channelAnalyseAceeptVo) {
        String text = "";
        if (channelAnalyseAceeptVo.getSearchText() != null)
            text = channelAnalyseAceeptVo.getSearchText();
        PageHelper.startPage(channelAnalyseAceeptVo.getPageNum(), channelAnalyseAceeptVo.getPageSize());
        List<Brand> brands = brandMapper.searchList(text);
        PageInfo<Brand> pageInfo = new PageInfo<>(brands);
        return pageInfo;
    }


    //TODO 待测试（redis版本）
    @Override
    public HashMap<String, Object> tagAnalyse(ChannelMapAceeptVo channelMapAceeptVo) {
        //1.根据条件的到位网点信息
        List<WdInfo> queryWd = getQueryWd(channelMapAceeptVo);

        //2.获取网点周边标签，并进行统计
        int total = 0;
        HashMap<String,Integer> hashMap = new HashMap<>();
        for (WdInfo wdInfo : queryWd) {
            List<String> wdTag = wdRedisStoreage.getWdTag(wdInfo);
            if (wdTag == null)
                continue;
            total++;
            for (String s : wdTag) {
                Integer integer = hashMap.get(s);
                if (integer == null)
                    hashMap.put(s,1);
                else
                    hashMap.put(s,integer+1);
            }

        }

        //3.封装统计值
        HashMap<String, Object> result = new HashMap<>();
        List<TagAnalyse> list = new ArrayList<>();
        for (String s : hashMap.keySet()) {
            TagAnalyse tagAnalyse = new TagAnalyse();
            tagAnalyse.setName(s);
            tagAnalyse.setCount(hashMap.get(s));
            BigDecimal bigDecimal = new BigDecimal((double) tagAnalyse.getCount() / total);
            double v = bigDecimal.setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue();
            tagAnalyse.setRadio(v);
            list.add(tagAnalyse);
        }

        //4.对结果集进行排序
        Collections.sort(list, new Comparator<TagAnalyse>() {
            @Override
            public int compare(TagAnalyse o1, TagAnalyse o2) {
                return o2.getCount() - o1.getCount();
            }
        });


        //5.分页返回结果集
        int i = 1;
        int start = 0,end = 6;
        while(start<list.size()){
            if(end>=list.size())
                end = list.size();
            result.put(""+i++,new ArrayList(list.subList(start,end)));
            start=end;
            end+=6;
        }
        result.put("pages",i-1);
        result.put("total",total);
        return result;
    }


    /**
     * 根据条件获取网点
     * */
    public List<WdInfo> getQueryWd(ChannelMapAceeptVo channelMapAceeptVo){
        //1.根据不同级别得到需要查询的 地区码
        List<String> addrCodeList = new ArrayList<>();
        if ("province".equals(channelMapAceeptVo.getRankType())) {
            //省码
            for (String s : channelMapAceeptVo.getAddrCode()) {
                String substring = s.substring(0, 2);
                addrCodeList.add(substring);
            }
        } else if ("city".equals(channelMapAceeptVo.getRankType())) {
            //省的所有市
            for (String s : channelMapAceeptVo.getAddrCode()) {
                String substring = s.substring(0, 2);
                addrCodeList.add(substring);
            }
        } else if ("zone".equals(channelMapAceeptVo.getRankType())) {
            //市的所有区
            for (String s : channelMapAceeptVo.getAddrCode()) {
                String substring = s.substring(0, 4);
                addrCodeList.add(substring);
            }
        } else if ("district".equals(channelMapAceeptVo.getRankType())){
            //区的所有街道
            for (String s : channelMapAceeptVo.getAddrCode()) {
                String substring = s.substring(0, 6);
                addrCodeList.add(substring);
            }
        }else
            return new ArrayList<>();


        //2.组装条件构造器找到网点信息网点
        //2.1 渠道、地区分类
        QueryWrapper<WdInfo> queryWrapper = new QueryWrapper<>();
        queryWrapper.in("wd_type_code", channelMapAceeptVo.getChannel()).and(originWdInfoQueryWrapper -> {
            for (String s : addrCodeList) {
                originWdInfoQueryWrapper.likeRight("addr_code", s).or();
            }
        });

        //搜索字段
        if (channelMapAceeptVo.getSearchText() != null && !channelMapAceeptVo.getSearchText() .trim().equals("")) {
            queryWrapper.and(originWdInfoQueryWrapper -> {
                originWdInfoQueryWrapper.like("wd_name", channelMapAceeptVo.getSearchText() );
            });
        }

        //城市等级分类
        if(channelMapAceeptVo.getCityTier() != null && channelMapAceeptVo.getCityTier().length > 0){
            List<String> tierCode = new ArrayList<>();
            for (String s : channelMapAceeptVo.getCityTier()) {
                List<String> list = cityTierListMap.get(s);
                if (list != null){
                    tierCode.addAll(list);
                }
            }
            queryWrapper.and(originWdInfoQueryWrapper -> {
                for (String s : tierCode) {
                    originWdInfoQueryWrapper.likeRight("addr_code", s).or();
                }
            });
        }


        List<WdInfo> wdInfos = wdInfoDao.selectList(queryWrapper);
        return wdInfos;
    }
}
